#!/usr/bin/perl

# Fuzzy Olfaction Gene Finder

#-----------------------------------------------------------------------
# Programmer:    <your name>
#
# SUMMARY:  
#
# INPUT:
#
# OUTPUT:
#
# Status:  
#
# Date Last Modified:
#    10/15/2007 (mdl, bdd) -- created "starter kit", including functions to use
#    10/??/2007 (   ) --


#-----------------------------------------------------------------------


#-----------------------------------------------------------------------

# be careful
use strict;
use warnings;

use File::Spec;	# needed for MacOS X


#-----------------------------------------------------------------------
#  CONSTANTS
#-----------------------------------------------------------------------
my $NUC_WIDTH = 70;	# 70 nucleotides per line when outputting sequences
my $AA_WIDTH  = 40;	# 40 amino acids per line when outputting sequences




# ------------- DECLARE VARIABLES ----------------------------------------------

my %codonMap;       # declare a hash table to hold the Amino Acid abbreviations
                    # corresponding to each possible triple of DNA (codon).
                    # Amino Acids are abgreviated with the ONE LETTER codes.
                    # (see the table at the bottom of this program for AA codes)
                    
my $filename;		# file holding gene sequence (FASTA format)

my $DNAsequence;       # will hold the DNA sequence to search
my $DNAsequenceLength; # to hold the total number of nucleotides to search








#-----------------------------------------------------------------------

# set up a hash table that will map codons (three-letter sequences) of DNA
# to their respective Amino Acid (AA) equivalent using ONE-letter AA codes
%codonMap = getAminoAcidLookUpTable();

$filename = "humanXM_000000.fna";

# get the DNA gene sequence found on Entrez(NCBI) and store it in a Perl string
$DNAsequence = getDNA($filename);

printSequence($DNAsequence, $NUC_WIDTH);












# ============ end of fuzzy.pl program ===============================================





#-------------------------------------------------------------------------------------
#--------------------------- Subroutines (functions)  --------------------------------
#-------------------------------------------------------------------------------------

#--------------\
# printTitle    \
#-----------------------------------------------------------------------
# Subroutine to print the title at the top of the output
#
# 3 arguments:  organismName, accession number, organism definition
#
# RETURNS: nothing

sub printTitle
{
   
   
   
   
   
} 
#----- end of printTitle ---------------------------------------------------------



#--------------\
# printSequence \
#-----------------------------------------------------------------------
# Subroutine to print a sequence, (second arg) characters per line
#
# 2 arguments:  (1) sequence to print (could be DNA, RNA, or Amino Acid sequence)
#               (2) an integer indicating how many characters to print per line
#
# RETURNS:  nothing 

sub printSequence
{
  my $someSequence = shift(@_);			# get sequence (first argument)
  my $N;								# YOU MUST get (second) number of nucleotides per line


  # note:  the 10 on the next line should be changed;  to what?
  print "======================================================================\n";
  while ($someSequence =~ m/(.{1,10})/g)
  {
      print "$1 \n";
  }
  print "======================================================================\n";
  
} 
#----- end of printSequence ---------------------------------------------------



#-----------------\
# countNucleotides \
#-----------------------------------------------------------------------
# Subroutine to COUNT and RETURN the number of nucleotides
# in the given sequence.
#
# two arguments:  sequence to search through and the nucleotide to count
#
# RETURNS: number of times the second argument appears in the first arg sequence
#
# A sample of how you might "call" this subroutine from your program:
#
#                  $numberOf_A = countNucleotides("ACGTACGT", "A");
#                  print "$numberOf_A";    # --> would print two(2), two As

sub countNucleotides
{
  my $someSequence;				 # YOU MUST get sequence sent, store in $someString
  my $whatBP;					 # YOU MUST get nucleotide sent, store in $whatBP

  my $howMany;

  # ignoring case (i), globally (g) find the number of matches (m)
  # of $whatBP that are found in $someSequence
  $howMany = () = $someSequence =~ m/$whatBP/ig ;

  return $howMany;
  
}
#----- end of countNucleotides ---------------------------------------------------




#-----------------------\
# generateRandomSequence \
#-----------------------------------------------------------------------
# Generate a random sequence of the same length as the first argument
# using the A,C,G,T nucleotide percentages in the 2nd, 3rd, 4th, and 5th 
# arguments.
#
# 5 arguments:  the original sequence (could be DNA, RNA, AA, etc),
#               then four arguments: % of As, %Cs, %Gs, and %Ts
#
# RETURNS:  random sequence  

sub generateRandomSequence
{














  
}
#----- end of generateRandomSequence --------------------------------------------------









# ============== Do NOT edit below this line; these functions are ready to use!


#------------------------------------------------------------------------------
# The subroutines getAminoAcidLookUpTable(), transcribe() and translate() are
# Copyright (c) 2002 Rex A. Dwyer.
#------------------------------------------------------------------------------


#------------------------\
# getAminoAcidLookUpTable \
#-----------------------------------------------------------------------
# Construct hash that maps codons to amino acids by reading the table
# from DATA lines at the end of the program, which have the form:
# Residue Codon1 Codon2 ...
#
# no arguments
#
# RETURNS: hash table of codon and their one-letter amino acid abbreviations

sub getAminoAcidLookUpTable
{
  my %codonMap;

  while (my $in = <DATA> )    # assigns next line of DATA to $in; fails if none
  {
    chomp($in);                   # remove line feed at end of $in
    my @codons = split " ",$in;
    my $residue = shift @codons;  # remove first item from @codons and assign
    foreach my $nnn (@codons) {
        $codonMap{$nnn} = $residue;
    }
  }
  
  return %codonMap;
  
}
#----- end of getAminoAcidLookUpTable ------------------------------------------



#-------\
# getDNA \
#-----------------------------------------------------------------------
# Subroutine to open a FASTA formatted file of DNA sequence, remove
# newline characters at the end of each line, and return as one big string.
#
# 1 argument:  name of file holding the DNA
#
# RETURNS:  file of DNA as one string
#
# A sample of how you might "call" this subroutine from your program
# to 'get' the DNA for chromosome III of the worm, C.elegans.
#
#                  $sequence = getDNA("worm_III.fna");

sub getDNA
{

  my $filename = shift(@_);  # get the filename sent
 
  my $firstline;             # holds header (first) line of file with name, etc
  my $DNAsequence;           # holds the entire sequence of DNA in the file
                             # to return
  my @lines;                 # used to hold ALL the lines from the DNA file
 
 
 # MacOS USERS TAKE NOTE
 # next two lines need to be uncommented for MacOS X (e.g., BBedit or TexWrangler)
 #my @path	= File::Spec->splitdir( File::Spec->rel2abs($0) );
 #$filename	= File::Spec->catfile( @path[0 .. $#path - 1], $filename );


  # try to open the file; if no file by that name, print a warning and exit
  open (FNAFILE, "$filename");
  if ( !($firstline = <FNAFILE>) ) {
    print "Can NOT open the file called: $filename \n";
    exit();
  }
  
  @lines = <FNAFILE>;                   # read all the remaining lines
  chomp @lines;                         # remove all the newlines at the end
  $DNAsequence = join( '', @lines);     # concatenate all the lines into 1 string
  
  # if we get to here, print SUCCESS and the first line of the FASTA file
  print "Successfully opened and read the file: \t $filename \n\n";
  print "HEADER LINE: \n";
  print "$firstline \n\n";
  
  $DNAsequence =~ tr/acgt/ACGT/;      # convert to upper-case nucleotide letters
  $DNAsequence =~ s/[^A-Z]//g;        # strip out line numbers and spaces
  
  return $DNAsequence;
  
}
#----- end of getDNA -------------------------------------------------------


#-----------\
# translate  \
#-----------------------------------------------------------------------
# translates (presumably pre-transcribed) DNA strings to proteins
#
# 1 argument: the DNA string to translate
#
# RETURNS: the protein string. (1-letter Amino Acid abbreviations)

sub translate
{
    my $dna = shift(@_);  # the DNA string to be translated.
    
    my $pro = "";
    while ( $dna =~ s/(...)// ) {
        $pro = $pro . $codonMap{$1};
    }
    return $pro;
}
#----- end of translate -------------------------------------------------------





## The lines below are not perl statements and are not executed as part of the
## program.  Instead, they are available to be read as input by the program
## using the file handle DATA. The subroutine getAminoAcidLookUpTable uses this
## table of amino acid abbreviations and corresponding (DNA) codons

__END__
A GCT GCC GCA GCG
R CGT CGC CGA CGG AGA AGG
N AAT AAC
D GAT GAC
C TGT TGC
Q CAA CAG
E GAA GAG
G GGT GGC GGA GGG
H CAT CAC
I ATT ATC ATA
L TTA TTG CTT CTC CTA CTG
K AAA AAG
M ATG
F TTT TTC
P CCT CCC CCA CCG
S TCT TCC TCA TCG AGT AGC
T ACT ACC ACA ACG
W TGG
Y TAT TAC
V GTT GTC GTA GTG
. TAA TAG TGA
